//
//  SetCardView.swift
//  Diamond
//
//  Created by Sameh Fakhouri on 10/28/19.
//  Copyright © 2019 Sameh Fakhouri. All rights reserved.
//

import UIKit

@IBDesignable class SetCardView: UIView {

    
    /***********************************************************/
    /*                                                         */
    /*  Properties                                             */
    /*                                                         */
    /***********************************************************/
    
    @IBInspectable var colorInt: Int = 0 {
        didSet {
            let num = colorInt % 3
            switch num {
            case 0: color = Colors.green
            case 1: color = Colors.red
            case 2: color = Colors.purple
            default: color = Colors.green
            }
        }
    }
    
    var color = Colors.green {
        didSet {
            setNeedsDisplay()
            setNeedsLayout()
        }
    }
    
    enum Colors {
        case green
        case red
        case purple
        
        static var all = [Colors.green, .red, .purple]
    }
    
    @IBInspectable var shadeInt: Int = 0 {
        didSet {
            let num = shadeInt % 3
            switch num {
            case 0: shade = Shades.outlined
            case 1: shade = Shades.striped
            case 2: shade = Shades.filled
            default: shade = Shades.outlined
            }
        }
    }
    
    var shade = Shades.outlined {
        didSet {
            setNeedsDisplay()
            setNeedsLayout()
        }
    }
    
    enum Shades {
        case outlined
        case striped
        case filled
        
        static var all = [Shades.outlined, .striped, .filled]
    }
    
    @IBInspectable var count: Int = 1 {
        didSet {
            setNeedsDisplay()
            setNeedsLayout()
        }
    }
    
    @IBInspectable var isSelected: Bool = true {
        didSet {
            setNeedsDisplay()
            setNeedsLayout()
        }
    }
    
    @IBInspectable var isMatched: Bool = false {
        didSet {
            setNeedsDisplay()
            setNeedsLayout()
        }
    }
    
    @IBInspectable var isMisMatched: Bool = false {
        didSet {
            setNeedsDisplay()
            setNeedsLayout()
        }
    }

    /***********************************************************/
    /*                                                         */
    /* Draw Diamonds                                           */
    /*                                                         */
    /***********************************************************/
    private func drawDiamond() -> UIBezierPath {
        let diamond = UIBezierPath()
        
        diamond.move(to: upperVertex)
        diamond.addLine(to: rightVertex)
        diamond.addLine(to: lowerVertex)
        diamond.addLine(to: leftVertex)
        diamond.addLine(to: upperVertex)
        diamond.close()
        
        return diamond
    }
    
    /***********************************************************/
    /*                                                         */
    /* Show Path                                               */
    /*                                                         */
    /***********************************************************/
    private func showPath(_ path: UIBezierPath) {
        var path = replicatePath(path)
        colorForPath.setStroke()
        path = shadePath(path)
        path.lineWidth = 2.0
        path.stroke()
        path.fill()
    }
    
    /***********************************************************/
    /*                                                         */
    /* Color Path                                              */
    /*                                                         */
    /***********************************************************/
    private var colorForPath: UIColor {
        switch color {
        case .green: return UIColor.green
        case .red: return UIColor.red
        case .purple: return UIColor.purple
        }
    }
    
    /***********************************************************/
    /*                                                         */
    /* Shade Path                                              */
    /*                                                         */
    /***********************************************************/
    private func shadePath(_ path: UIBezierPath) -> UIBezierPath {
        let shadedPath = UIBezierPath()
        shadedPath.append(path)
        
        switch shade {
        case .filled:
            colorForPath.setFill()
        case .striped:
            UIColor.clear.setFill()
            shadedPath.addClip()
            var start = CGPoint(x:0.0, y:0.0)
            var end = CGPoint(x: self.bounds.size.width, y: 0.0)
            let dy: CGFloat = self.bounds.height / 10.0
            while start.y <= self.bounds.height {
                shadedPath.move(to: start)
                shadedPath.addLine(to: end)
                start.y += dy
                end.y += dy
            }
        case .outlined:
            UIColor.clear.setFill()
        }
        
        return shadedPath
    }
    
    /***********************************************************/
    /*                                                         */
    /* Show Path                                               */
    /*                                                         */
    /***********************************************************/
    private func replicatePath(_ path: UIBezierPath) -> UIBezierPath {
        let replicatedPath = UIBezierPath()
        
        if count == 1 {
            replicatedPath.append(path)
        } else if count == 2 {
            let leftPath = UIBezierPath()
            leftPath.append(path)
            let leftPathTransform = CGAffineTransform(
            translationX: leftTwoPathTranslation.x, y: leftTwoPathTranslation.y)
            leftPath.apply(leftPathTransform)
            
            let rightPath = UIBezierPath()
            rightPath.append(path)
            let rightPathTransform = CGAffineTransform(
            translationX: rightTwoPathTranslation.x, y: rightTwoPathTranslation.y)
            rightPath.apply(rightPathTransform)
            
            replicatedPath.append(leftPath)
            replicatedPath.append(rightPath)
        } else if count == 3 {
            let leftPath = UIBezierPath()
            leftPath.append(path)
            let leftPathTransform = CGAffineTransform(
                translationX: leftThreePathTranslation.x, y: leftThreePathTranslation.y)
            leftPath.apply(leftPathTransform)
            
            let rightPath = UIBezierPath()
            rightPath.append(path)
            let rightPathTransform = CGAffineTransform(
                translationX: rightThreePathTranslation.x, y: rightThreePathTranslation.y)
            rightPath.apply(rightPathTransform)
            
            replicatedPath.append(leftPath)
            replicatedPath.append(path)
            replicatedPath.append(rightPath)
        }
        
        return replicatedPath
    }
    
    
    /***********************************************************/
    /*                                                         */
    /* draw                                                    */
    /*                                                         */
    /***********************************************************/
    override func draw(_ rect: CGRect) {
        let roundedRect = UIBezierPath(roundedRect: bounds, cornerRadius: cornerRadius)
        roundedRect.addClip()
        
        if isSelected {
            roundedRect.lineWidth = 5.0
            UIColor.black.setStroke()
        } else if isMatched {
            roundedRect.lineWidth = 5.0
            UIColor.green.setStroke()
        } else if isMisMatched {
            roundedRect.lineWidth = 5.0
            UIColor.red.setStroke()
        } else {
            roundedRect.lineWidth = 5.0
            UIColor.white.setStroke()
        }
        
        UIColor.white.setFill()
        roundedRect.fill()
        roundedRect.stroke()
        
        let path = UIBezierPath()
        path.append(drawDiamond())
        
        showPath(path)
    }
}

extension SetCardView {
    /***********************************************************/
    /*                                                         */
    /* Corner Radius                                           */
    /*                                                         */
    /***********************************************************/
    
    private var cornerRadius: CGFloat {
        return bounds.size.height * 0.06
    }
}

extension SetCardView {
    /***********************************************************/
    /*                                                         */
    /* Diamond Constants                                       */
    /*                                                         */
    /***********************************************************/
    
    private struct DiamondRatios {
        static let upperVertexOffsetPercentage:  CGFloat = 0.10
        static let lowerVertexOffsetPercentage:  CGFloat = 0.10
        static let widthPercentage:              CGFloat = 0.20
    }
    
    private var upperVertex: CGPoint {
        return CGPoint(x: self.bounds.size.width/2.0,
                       y: self.bounds.size.height * DiamondRatios.upperVertexOffsetPercentage)
    }
    
    private var lowerVertex: CGPoint {
        return CGPoint(x: self.bounds.width/2.0,
                       y: self.bounds.height - (self.bounds.size.height * DiamondRatios.lowerVertexOffsetPercentage))
    }
    
    private var leftVertex: CGPoint {
        return CGPoint(x: self.bounds.size.width/2.0 - (self.bounds.size.width * DiamondRatios.widthPercentage / 2.0),
                       y: self.bounds.size.height/2.0)
    }
    
    private var rightVertex: CGPoint {
        return CGPoint(x: self.bounds.size.width/2.0 + (self.bounds.size.width * DiamondRatios.widthPercentage / 2.0),
                       y: self.bounds.size.height/2.0)
    }
}

extension SetCardView {
    /***********************************************************/
    /*                                                         */
    /* Path Replication Constants                              */
    /*                                                         */
    /***********************************************************/
    
    private var leftTwoPathTranslation: CGPoint {
        return CGPoint(x: self.bounds.size.width * -0.15,
                       y: 0.0)
    }
    
    private var rightTwoPathTranslation: CGPoint {
        return CGPoint(x: self.bounds.size.width * 0.15,
                       y: 0.0)
    }
    
    private var leftThreePathTranslation: CGPoint {
        return CGPoint(x: self.bounds.size.width * -0.25,
                       y: 0.0)
    }
    
    private var rightThreePathTranslation: CGPoint {
        return CGPoint(x: self.bounds.size.width * 0.25,
                       y: 0.0)
    }
}

